relationPriority = {
    "fixed": 9,
    "goeswith": 9,
    "parataxis": 9,
    "appos": 9,
    "amod": 10,
    "compound": 10,
    "compound:prt": 10,
    "nummod": 10,
    "expl": 10,
    "det": 15,
    "det:predet": 15,
    "cc:preconj": 25,
    "cc": 25,
    "acl:relcl": 16,
    "acl": 16,
    "cop": 40,
    "obj": 40,
    "case": 20,
    "mark": 15,
    "xcomp": 10,
    "nmod:poss": 10,
    "nmod:npmod": 40,
    "nmod:tmod": 40,
    "nmod:count": 40,
    "nmod": 40,
    "obl": 40,
    "iobj": 40,
    "obl:npmod": 40,
    "obl:tmod": 42,
    "advmod": 20,
    "advmod:count": 20,
    "advcl": 40,
    "ccomp": 10,
    "aux:pass": 45,
    "aux": 45,
    "dep": 45,
    "conj": 45,
    "conj:vp": 45,
    "csubj": 60,
    "csubj:pass": 60,
    "nsubj": 60,
    "nsubj:pass": 60,
    "nsubj:wh": 60,
    "conj:sent": 80,
    "punct": 90,
    "root": 90,
    "discourse": 80
}
